function [data] = select_data(signal, hypno, art, state, min_state_l, samp_rate)
    % Return EEG signal based on no artifacts and state of the sleep
    % 
    % [data] = select_data(signal, hypno, art, state, min_state_l, samp_rate)
    %
    % Parameters:
    %       signal      - matrix of the EEG signal
    %       hypno       - hypnogram 
    %       art         - matrix represented artifacts occurence
    %       state       - state of the sleep 
    %                     0 - wake, 
    %                     1 - S1,
    %                     2 - S2,
    %                     3 - S3,
    %                     4 - S4,
    %                     5 - REM,
    %                     6 - movement time
    %                     9 - unscored (artifact)
    %       min_state_l - minimum length of the sleep state in 30s units 
    %       samp_rate   - sampling rate of the collecting signal
    
    state_list = containers.Map([0, 1, 2, 3, 4, 5, 6, 9], {'wake', 'S1', 'S2', 'S3' , 'S4', 'REM', ...
             'movement time', 'unscored (artifact)'});
    
    
    
    %%  Find each desired sleep state which last at least "min_state_l" without artifacts
    
    % Find localization of the desired sleep state
    ind_hypno = find(hypno == state);
    
    % Find all data without artifact in every source 
    ind_art = all(ind2sub(size(art),art == 0), 2);
    
    % Check for artifacts in desired state 
    temp = [];% zeros( size( ind_hypno, 2), 1); % allocation for 
    
    for i = 1:size(ind_hypno, 2)
        
        % Check if an artifact is presented in the particular state  
        % (30s time window)
        
        if all(ind_art( ind_hypno(i) * 10 : ind_hypno(i) * 10 + 9) == 1)
            temp = [temp ind_hypno(i)];
        end
    end
    
    % Find all the desired states without artifacts
    ind_hypno = temp; 
    
    a = diff(ind_hypno);    % take the difference from last value
    b = find([a inf]>1);    % find where it is more than 1 ('jump')
    c = diff([0 b]);        % length of the sequences
    d = cumsum(c);          % endpoints of the sequences
    
    temp_ind = find(c > 5);
    
    fprintf('\nThere are %2d "%s" phases longer than %2.2f min and without artifacts.\n', ...
        size(temp_ind, 2), state_list(state), (min_state_l * 30) / 60);
    
    pause()
    
    % Find [START END] of the states 
    ind_final = zeros(size(temp_ind, 2), 2);
    for i = 1:size(c,2)
        if c(i) < 5
            continue
        end
        ind_final(i,:) = [ind_hypno(d(i) - (c(i)-1)), ind_hypno(d(i))];
    end

    ind_final = ind_final(all(ind2sub(size(ind_final),ind_final ~= 0), 2) , : );
    
    %% Subset desired data 
    
    fprintf('\nThere are following time windows where the subjects were in desired state %s\n\n', ...
        state_list(state));
    for i = 1:size(ind_final)
        st_start    = secs2hms(ind_final(i,1) * 30);
        st_end      = secs2hms(ind_final(i,2) * 30);
        fprintf(' %d.phase: %s   --  %s \n', i, st_start, st_end);
    end

    pause()
    
    temp = input('\nWhich phase do you want to select? '); 
    phase_start = ind_final(temp,1) * samp_rate * 30; 
    phase_end   = ind_final(temp,2) * samp_rate * 30; 

    % Finally select the data
    data = signal(phase_start : phase_end , :);
    
    fprintf('\n You choosed %d.phase of "%s" sleep stage.', temp, state_list(state));
    fprintf('\n Its duration is between %s  and  %s.\n', ...
        secs2hms(ind_final(temp,1) * 30), secs2hms(ind_final(temp,2) * 30));
end